@@ -8,6 +8,10 @@ extern int gdbg;
 #endif 
 #endif 
 */
+
+#ifndef __PROXYSQL_DEBUG_H
+#define __PROXYSQL_DEBUG_H
+
 #ifdef DEBUG
 #define PROXY_TRACE() { proxy_debug(PROXY_DEBUG_GENERIC,10,"TRACE\n"); }
 //#define PROXY_TRACE2() { proxy_info("TRACE\n"); }
@@ -41,9 +45,20 @@ extern int gdbg;
 		time(&__timer); \
 		localtime_r(&__timer, &__tm_info); \
 		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", &__tm_info); \
-		proxy_error_func("%s %s:%d:%s(): [ERROR] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+		proxy_error_func(0, "%s %s:%d:%s(): [ERROR] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
 	} while(0)
 
+#define proxy_error2(ecode, fmt, ...) \
+    do { \
+        time_t __timer; \
+        char __buffer[30]; \
+        struct tm __tm_info; \
+        time(&__timer); \
+        localtime_r(&__timer, &__tm_info); \
+        strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", &__tm_info); \
+        proxy_error_func(ecode, "%s %s:%d:%s(): [ERROR] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+    } while(0)
+
 #define proxy_error_inline(fi, li, fu, fmt, ...) \
 	do { \
 		time_t __timer; \
@@ -52,7 +67,7 @@ extern int gdbg;
 		time(&__timer); \
 		localtime_r(&__timer, &__tm_info); \
 		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", &__tm_info); \
-		proxy_error_func("%s %s:%d:%s(): [ERROR] " fmt, __buffer, fi, li, fu , ## __VA_ARGS__); \
+		proxy_error_func(0, "%s %s:%d:%s(): [ERROR] " fmt, __buffer, fi, li, fu , ## __VA_ARGS__); \
 	} while(0)
 /*
 #else
@@ -79,8 +94,20 @@ extern int gdbg;
 		time(&__timer); \
 		__tm_info = localtime(&__timer); \
 		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
-		proxy_error_func("%s %s:%d:%s(): [WARNING] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+		proxy_error_func(0, "%s %s:%d:%s(): [WARNING] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
 	} while(0)
+
+#define proxy_warning2(ecode, fmt, ...) \
+	do { \
+		time_t __timer; \
+		char __buffer[25]; \
+		struct tm *__tm_info; \
+		time(&__timer); \
+		__tm_info = localtime(&__timer); \
+		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
+		proxy_error_func(ecode, "%s %s:%d:%s(): [WARNING] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+	} while(0)
+
 /*
 #else
 #define proxy_warning(fmt, ...) \
@@ -104,7 +131,18 @@ extern int gdbg;
 		time(&__timer); \
 		__tm_info = localtime(&__timer); \
 		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
-		proxy_error_func("%s %s:%d:%s(): [INFO] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+		proxy_error_func(0, "%s %s:%d:%s(): [INFO] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
+	} while(0)
+
+#define proxy_info2(ecode, fmt, ...) \
+	do { \
+		time_t __timer; \
+		char __buffer[25]; \
+		struct tm *__tm_info; \
+		time(&__timer); \
+		__tm_info = localtime(&__timer); \
+		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
+		proxy_error_func(ecode, "%s %s:%d:%s(): [INFO] " fmt, __buffer, __FILE__, __LINE__, __func__ , ## __VA_ARGS__); \
 	} while(0)
 #else
 #define proxy_info(fmt, ...) \
@@ -115,7 +153,18 @@ extern int gdbg;
 		time(&__timer); \
 		__tm_info = localtime(&__timer); \
 		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
-    proxy_error_func("%s [INFO] " fmt , __buffer , ## __VA_ARGS__); \
+		proxy_error_func(0, "%s [INFO] " fmt , __buffer , ## __VA_ARGS__); \
+	} while(0)
+
+#define proxy_info2(fmt, ...) \
+	do { \
+		time_t __timer; \
+		char __buffer[25]; \
+		struct tm *__tm_info; \
+		time(&__timer); \
+		__tm_info = localtime(&__timer); \
+		strftime(__buffer, 25, "%Y-%m-%d %H:%M:%S", __tm_info); \
+		proxy_error_func(ecode, "%s [INFO] " fmt , __buffer , ## __VA_ARGS__); \
 	} while(0)
 #endif
 
@@ -131,4 +180,25 @@ extern int gdbg;
 		} \
 	} while(0)
 
+struct p_debug_dyn_counter {
+	enum metric {
+		proxysql_message_count = 0,
+		__size
+	};
+};
 
+struct debug_metrics_map_idx {
+	enum index {
+		dyn_counters,
+	};
+};
+
+class SQLite3_result;
+SQLite3_result* proxysql_get_message_stats(bool reset=false);
+
+/**
+ * @brief Initializes the prometheus metrics contained in 'debug.cpp'.
+ */
+void proxysql_init_debug_prometheus_metrics();
+
+#endif
