@@ -33,13 +33,10 @@
 
 #include <openssl/x509v3.h>
 
-// Minimal headers for exporting metrics using prometheus
-#include <prometheus/counter.h>
-#include <prometheus/exposer.h>
-#include <prometheus/registry.h>
-
 #include <sys/mman.h>
 
+#include <uuid/uuid.h>
+
 /*
 extern "C" MySQL_LDAP_Authentication * create_MySQL_LDAP_Authentication_func() {
 	return NULL;
@@ -111,6 +108,10 @@ static char * main_check_latest_version() {
 	curl_global_init(CURL_GLOBAL_ALL);
 	curl_handle = curl_easy_init();
 	curl_easy_setopt(curl_handle, CURLOPT_URL, "https://www.proxysql.com/latest");
+	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYPEER, 0L);
+	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYHOST, 0L);
+	curl_easy_setopt(curl_handle, CURLOPT_SSL_VERIFYSTATUS, 0L);
+	curl_easy_setopt(curl_handle, CURLOPT_RANGE, "0-31");
 	curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
 	curl_easy_setopt(curl_handle, CURLOPT_WRITEDATA, (void *)&chunk);
 
@@ -337,7 +338,7 @@ void * __qc;
 void * __mysql_thread;
 void * __mysql_threads_handler;
 void * __query_processor;
-//void * __mysql_auth; 
+//void * __mysql_auth;
 
 
 
@@ -464,6 +465,7 @@ void * mysql_shared_query_cache_funct(void *arg) {
 
 void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 	GloVars.errorlog = NULL;
+	GloVars.pid = NULL;
 	GloVars.parse(argc,argv);
 	GloVars.process_opts_pre();
 	GloVars.restart_on_missing_heartbeats = 10; // default
@@ -499,6 +501,41 @@ void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 				GloVars.errorlog = strdup(errorlog_path.c_str());
 			}
 		}
+		if (root.exists("uuid")==true) {
+			string uuid;
+			bool rc;
+			rc=root.lookupValue("uuid", uuid);
+			if (rc==true) {
+				uuid_t uu;
+				if (uuid_parse(uuid.c_str(), uu)==0) {
+					if (GloVars.uuid == NULL) {
+						// it is not set yet, that means it wasn't specified on the cmdline
+						GloVars.uuid = strdup(uuid.c_str());
+					}
+				} else {
+					proxy_error("The config file is configured with an invalid UUID: %s\n", uuid.c_str());
+				}
+			}
+		}
+		// if cluster_sync_interfaces is true, interfaces variables are synced too
+		if (root.exists("cluster_sync_interfaces")==true) {
+			bool value_bool;
+			bool rc;
+			rc=root.lookupValue("cluster_sync_interfaces", value_bool);
+			if (rc==true) {
+				GloVars.cluster_sync_interfaces=value_bool;
+			} else {
+				proxy_error("The config file is configured with an invalid cluster_sync_interfaces\n");
+			}
+		}
+		if (root.exists("pidfile")==true) {
+			string pidfile_path;
+			bool rc;
+			rc=root.lookupValue("pidfile", pidfile_path);
+			if (rc==true) {
+				GloVars.pid = strdup(pidfile_path.c_str());
+      }
+    }
 		if (root.exists("sqlite3_plugin")==true) {
 			string sqlite3_plugin;
 			bool rc;
@@ -587,8 +624,10 @@ void ProxySQL_Main_process_global_variables(int argc, const char **argv) {
 		sprintf(GloVars.errorlog,"%s/%s",GloVars.datadir, (char *)"proxysql.log");
 	}
 
-	GloVars.pid=(char *)malloc(strlen(GloVars.datadir)+strlen((char *)"proxysql.pid")+2);
-	sprintf(GloVars.pid,"%s/%s",GloVars.datadir, (char *)"proxysql.pid");
+	if (GloVars.pid == NULL) {
+		GloVars.pid=(char *)malloc(strlen(GloVars.datadir)+strlen((char *)"proxysql.pid")+2);
+		sprintf(GloVars.pid,"%s/%s",GloVars.datadir, (char *)"proxysql.pid");
+	}
 
 	if (GloVars.__cmd_proxysql_initial==true) {
 		std::cerr << "Renaming database file " << GloVars.admindb << endl;
@@ -846,6 +885,9 @@ void ProxySQL_Main_shutdown_all_modules() {
 	}
 
 	{
+#ifdef TEST_WITHASAN
+		pthread_mutex_lock(&GloAdmin->sql_query_global_mutex);
+#endif
 		cpu_timer t;
 		delete GloAdmin;
 #ifdef DEBUG
@@ -874,7 +916,7 @@ void ProxySQL_Main_init() {
 	glovars.has_debug=false;
 #endif /* DEBUG */
 //	__thr_sfp=l_mem_init();
-
+	proxysql_init_debug_prometheus_metrics();
 }
 
 
@@ -1036,7 +1078,11 @@ void ProxySQL_Main_init_phase3___start_all() {
 #endif
 	}
 
-	do { /* nothing */ } while (load_ != 1);
+	do { /* nothing */
+#ifdef DEBUG
+		usleep(5+rand()%10);
+#endif
+	} while (load_ != 1);
 	load_ = 0;
 	__sync_fetch_and_add(&GloMTH->status_variables.threads_initialized, 1);
 
@@ -1168,7 +1214,7 @@ bool ProxySQL_daemonize_phase2() {
 	//daemon_log(LOG_INFO, "Starting ProxySQL\n");
 	//daemon_log(LOG_INFO, "Sucessfully started");
 	proxy_info("Starting ProxySQL\n");
-	proxy_info("Sucessfully started\n");
+	proxy_info("Successfully started\n");
 	return true;
 }
 
@@ -1464,6 +1510,10 @@ __start_label:
 #ifdef DEBUG
 		std::cerr << "WARNING: this is a DEBUG release and can be slow or perform poorly. Do not use it in production" << std::endl;
 #endif
+	proxy_info("For information about products and services visit: https://proxysql.com/\n");
+	proxy_info("For online documentation visit: https://proxysql.com/documentation/\n");
+	proxy_info("For support visit: https://proxysql.com/services/support/\n");
+	proxy_info("For consultancy visit: https://proxysql.com/services/consulting/\n");
 
 	{
 		unsigned int missed_heartbeats = 0;
@@ -1541,11 +1591,15 @@ __start_label:
 					proxy_error("Watchdog: %u threads missed a heartbeat\n", threads_missing_heartbeat);
 					missed_heartbeats++;
 					if (missed_heartbeats >= (unsigned int)GloVars.restart_on_missing_heartbeats) {
+#ifdef RUNNING_ON_VALGRIND
+						proxy_error("Watchdog: reached %u missed heartbeats. Not aborting because running under Valgrind\n", missed_heartbeats);
+#else
 						if (GloVars.restart_on_missing_heartbeats) {
 							proxy_error("Watchdog: reached %u missed heartbeats. Aborting!\n", missed_heartbeats);
 							proxy_error("Watchdog: see details at https://github.com/sysown/proxysql/wiki/Watchdog\n");
 							assert(0);
 						}
+#endif
 					}
 				} else {
 					missed_heartbeats = 0;
